/*
								+--------------------------------+
								|                                |
								|  ***   Tray icon helper   ***  |
								|                                |
								|  Copyright  -tHE SWINe- 2010  |
								|                                |
								|           TrayIcon.h           |
								|                                |
								+--------------------------------+
*/

#ifndef __TRAY_ICON_INCLUDED
#define __TRAY_ICON_INCLUDED

#include <windows.h>
#include <shellapi.h>

#define WM_TRAYICON_NOTIFY (WM_USER+1)

#define NIF_INFO        0x00000010

// popup balloon icons
#define NIIF_NONE 0x00000000
#define NIIF_INFO 0x00000001
#define NIIF_WARNING 0x00000002
#define NIIF_ERROR 0x00000003

/**
 *	@def strcpy_ss
 *	@brief safe strcpy to static array.
 *	@param[out] d is destination, must be static array)
 *	@param[in] s is source string, can be 0; in case
 *		it is longer than capacity of d, it gets trimmed.
 */
#define strcpy_ss(d,s) \
	do { \
		size_t n_dl = sizeof(d) / sizeof((d)[0]); \
		const char *ps = s; \
		if(!ps) ps = ""; \
		strncpy(d, ps, n_dl); \
		(d)[n_dl] = 0; \
	} while(0)

class CTrayIcon {
protected:
	HWND m_h_parent;
	int m_n_id;

public:
	/**
	 *	@brief creates tray icon
	 *
	 *	@param[in] h_parent is window, associated with tray icon
	 *	@param[in] n_id is tray icon id (use in case there are more tray icons in your app)
	 *	@param[in] p_s_title is text, displayed when user points cursor at the icon
	 *	@param[in] h_icon is icon image
	 *	@param[in] n_message is icon event notification message
	 *
	 *	Use the following code to process messages from tray icon:
	 *@code
	 *	case WM_TRAYICON_NOTIFY:
	 *		{
	 *			int n_message = LOWORD(n_l_param);
	 *			int n_icon_id = HIWORD(n_l_param); // corresponds to n_id in CTrayIcon::CTrayIcon()
	 *			int n_mouse_x = LOWORD(n_w_param); // or GET_X_LPARAM
	 *			int n_mouse_y = HIWORD(n_w_param); // or GET_Y_LPARAM
	 *			if(n_message == WM_RBUTTONDOWN)
	 *				MessageBox(h_wnd, "You clicked tray icon!", "Tray alert", MB_OK);
	 *		}
	 *		break;
	 *@endcode
	 */
	CTrayIcon(HWND h_parent, int n_id, const char *p_s_title,
		HICON h_icon, int n_message = WM_TRAYICON_NOTIFY)
		:m_h_parent(0), m_n_id(-1)
	{
		Create(h_parent, n_id, p_s_title, h_icon, n_message);
	}

	CTrayIcon()
		:m_h_parent(0)
	{}

	~CTrayIcon()
	{
		if(m_h_parent) {
			NOTIFYICONDATA IconData = {0};
			IconData.cbSize = sizeof(IconData);
			IconData.uFlags = 0;
			IconData.uID = m_n_id;
			IconData.hWnd = m_h_parent;
			Shell_NotifyIcon(NIM_DELETE, &IconData);
		}
	}

	void Create(HWND h_parent, int n_id, const char *p_s_title,
		HICON h_icon, int n_message = WM_TRAYICON_NOTIFY)
	{
		if(m_h_parent)
			return; // allready have one

		m_h_parent = h_parent;
		m_n_id = n_id;

		NOTIFYICONDATA IconData = {0};
		IconData.cbSize = sizeof(IconData);
		IconData.uFlags = ((h_icon == NULL)? 0 : NIF_ICON) | ((p_s_title == 0)? 0 : NIF_TIP) | NIF_MESSAGE;
		IconData.uID = n_id;
		IconData.hWnd = m_h_parent;
		IconData.uCallbackMessage = n_message;
		IconData.hIcon = h_icon;
		strcpy_ss(IconData.szTip, p_s_title);
		Shell_NotifyIcon(NIM_ADD, &IconData);
	}

	int n_Id() const
	{
		return m_n_id;
	}

	void SetTitle(const char *p_s_title)
	{
		NOTIFYICONDATA IconData = {0};
		IconData.cbSize = sizeof(IconData);
		IconData.uFlags = NIF_TIP;
		IconData.uID = m_n_id;
		IconData.hWnd = m_h_parent;
		strcpy_ss(IconData.szTip, p_s_title);
		Shell_NotifyIcon(NIM_MODIFY, &IconData);
	}

	void SetIcon(HICON h_icon)
	{
		NOTIFYICONDATA IconData = {0};
		IconData.cbSize = sizeof(IconData);
		IconData.uFlags = NIF_ICON;
		IconData.uID = m_n_id;
		IconData.hWnd = m_h_parent;
		IconData.hIcon = h_icon;
		Shell_NotifyIcon(NIM_MODIFY, &IconData);
	}

	void ShowBalloon(const char *p_s_text, const char *p_s_title,
		int n_icon = NIIF_INFO, int n_timeout_msec = 10000)
	{
		Current_NOTIFYICONDATAA IconData2 = {0};
		IconData2.cbSize = sizeof(IconData2);
		IconData2.uFlags = NIF_INFO;
		IconData2.uID = m_n_id;
		IconData2.hWnd = m_h_parent;
		strcpy_ss(IconData2.szInfo, p_s_text);
		strcpy_ss(IconData2.szInfoTitle, p_s_title);
		IconData2.dwInfoFlags = n_icon;
		IconData2.uTimeout = n_timeout_msec;
		Shell_NotifyIcon(NIM_MODIFY, (NOTIFYICONDATA*)&IconData2);
	}

protected:
	struct Current_NOTIFYICONDATAA {
		DWORD cbSize;
		HWND hWnd;
		UINT uID;
		UINT uFlags;
		UINT uCallbackMessage;
		HICON hIcon;

		TCHAR szTip[128];
		DWORD dwState;
		DWORD dwStateMask;
		TCHAR szInfo[256];
		UINT  uTimeout;
		TCHAR szInfoTitle[64];
		DWORD dwInfoFlags;
	};
};

#endif //__TRAY_ICON_INCLUDED
